import { Component, EventEmitter, OnInit, Output } from '@angular/core';
import { ReviewManagementService } from 'app/admin/review-management/review-management.service';
import { Review } from 'app/admin/review-management/review.model';
import { ReviewRequest } from 'app/admin/review-management/reviewRequest.model';
import { UserManagementService } from 'app/admin/user-management/service/user-management.service';
import { User } from 'app/admin/user-management/user-management.model';
import { Account } from 'app/core/auth/account.model';
import { AccountService } from 'app/core/auth/account.service';
import { ExerciseService } from 'app/exercise/service/exercise.service';
import { SearchService } from 'app/search/service/search-service';
import { Exercise, searchResultToExercise } from 'app/shared/model/exercise.model';
import { SearchResultDTO } from 'app/shared/model/search/search-result-dto.model';
import { SearchResultsDTO } from 'app/shared/model/search/search-results-dto.model';

@Component({
  selector: 'jhi-peer-reviewing',
  templateUrl: './peer-reviewing.component.html',
  styleUrls: ['./peer-reviewing.component.scss'],
})
export class PeerReviewingComponent implements OnInit {
  isChecked = false;
  currentAccount: Account | null = null;
  user: User | null = null;
  results: Exercise[] = [];
  allExercises: Exercise[] = [];
  selectedExercise: Exercise | undefined;
  reviews: Review[] = [];
  otherReviews: Review[] = [];
  userReviews: Review[] = [];
  selectedReview = new Review('', [''], [''], ['']);
  options = ['IMPROVEMENTS_REQUESTED', 'REVIEW_REJECTED', 'REVIEW_COMPLETED'];
  selectedResult?: Exercise;
  selectedStatus = '';
  showOtherHistory: boolean[] = [];
  showHistory: boolean[] = [];

  @Output() exerciseSelectionEvent = new EventEmitter<Exercise>();

  constructor(
    private userService: UserManagementService,
    private accountService: AccountService,
    private searchService: SearchService,
    private reviewManagementService: ReviewManagementService,
    private exerciseService: ExerciseService
  ) {}

  ngOnInit(): void {
    this.accountService.identity().subscribe(account => (this.currentAccount = account));
    this.getUserFromLogin();
    this.searchForExercises();
    this.loadAll();
  }

  getUserFromLogin(): void {
    if (this.currentAccount != null && this.currentAccount.login != null) {
      this.userService.getLoggedInUser().subscribe(val => {
        this.user = val;
      });
    }
  }

  loadAll(): void {
    this.reviewManagementService.getAllByUser().subscribe(reviews => {
      this.reviews = reviews;
      this.userReviews = reviews.filter(review => review.requestedBy == this.user!.email);
      this.showHistory = Array.from(Array(this.userReviews.length).keys()).map(() => false);
      this.otherReviews = reviews.filter(review => !(review.requestedBy == this.user!.email));
      this.showOtherHistory = Array.from(Array(this.otherReviews.length).keys()).map(() => false);
    });
  }

  setReviewerStatusForLogin(): void {
    if (this.user != null && this.currentAccount != null && this.user.login == this.currentAccount.login) {
      this.userService.updateReviewerStatus(!this.user.reviewingEnabled).subscribe();
    }
  }

  searchForExercises(): void {
    this.searchService.getResultsForLoggedInUser().subscribe(
      (data: SearchResultsDTO) => {
        const searchResults = data.searchResult.map(searchResultToExercise);
        this.results = this.results
          .concat(searchResults)
          .filter((value, index, self) => self.findIndex(t => t.title === value.title) === index);
      },
      () => alert('Search failed')
    );
    this.searchService.getAllResources().subscribe(
      (data: SearchResultDTO[]) => {
        const searchResults = data.map(searchResultToExercise);
        this.allExercises = this.allExercises
          .concat(searchResults)
          .filter((value, index, self) => self.findIndex(t => t.title === value.title) === index);
      },
      () => (this.allExercises = [])
    );
  }

  shareExercise(exercise: Exercise) {
    this.selectedExercise = exercise;
  }

  selectExercise(gitlabURL: string): Exercise {
    const exercise: Exercise = this.exerciseService.populateExerciseWithData(
      this.allExercises.find(exercise => exercise.gitlabURL == gitlabURL)!
    );
    return exercise;
  }

  requestReview(): void {
    if (this.currentAccount) {
      this.searchService
        .requestReview(
          new ReviewRequest(
            this.selectedExercise!.title,
            this.selectedExercise!.originalResult.exerciseId,
            [this.currentAccount.email],
            this.selectedExercise!.gitlabURL
          )
        )
        .subscribe(() => this.loadAll());
    }
  }

  resourceImproved(review: Review): void {
    review.status![0] = 'REVIEW_IMPROVED';
    this.reviewManagementService.notifyReviewer(review).subscribe(() => this.loadAll());
  }

  updateReview(): void {
    this.selectedReview.status![0] = this.selectedStatus;
    this.reviewManagementService.updateByUser(this.selectedReview).subscribe(() => this.loadAll());
  }

  deleteReview(review: Review) {
    review.status![0] = 'DELETION_REQUESTED';
    this.reviewManagementService.updateByUser(review).subscribe(() => this.loadAll());
  }

  toggleHistory(index: number) {
    this.showHistory[index] = !this.showHistory[index];
  }

  isHistoryVisible(index: number): boolean {
    return this.showHistory[index];
  }

  toggleOtherHistory(index: number) {
    this.showOtherHistory[index] = !this.showOtherHistory[index];
  }

  isOtherHistoryVisible(index: number): boolean {
    return this.showOtherHistory[index];
  }
}
