package at.ac.uibk.gitsearch.service;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.assertEquals;

import at.ac.uibk.gitsearch.IntegrationTest;
import at.ac.uibk.gitsearch.domain.Review;
import at.ac.uibk.gitsearch.domain.Statistics;
import at.ac.uibk.gitsearch.domain.User;
import at.ac.uibk.gitsearch.domain.enumeration.ReviewStatus;
import at.ac.uibk.gitsearch.repository.jpa.ReviewJPARepository;
import at.ac.uibk.gitsearch.repository.jpa.ReviewRatingJPARepository;
import at.ac.uibk.gitsearch.repository.jpa.StatisticsRepository;
import at.ac.uibk.gitsearch.security.AuthoritiesConstants;
import at.ac.uibk.gitsearch.service.dto.ReviewDTO;
import at.ac.uibk.gitsearch.service.dto.ReviewRatingDTO.ReviewStatisticsDTO;
import at.ac.uibk.gitsearch.service.dto.ReviewRequest;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.Timeout;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.transaction.annotation.Transactional;

@IntegrationTest
public class ReviewServiceTest {

    @Autowired
    private ReviewJPARepository reviewRepository;

    @Autowired
    private ReviewRatingJPARepository reviewRatingRepository;

    @Autowired
    private ReviewService reviewService;

    @Autowired
    private UserService userService;

    @Autowired
    private StatisticsRepository statisticsRepository;

    @BeforeEach
    @WithMockUser(authorities = AuthoritiesConstants.ADMIN, username = "admin")
    public void tearDown() {
        reviewRatingRepository.deleteAll();
        reviewRepository.deleteAll();
        List<String> reviewers = new ArrayList<>();
        reviewers.add("user@localhost");

        reviewService.createReview(new ReviewRequest("resource", "codeability.at", "1", reviewers));
    }

    @Test
    @Transactional
    @WithMockUser(authorities = AuthoritiesConstants.ADMIN, username = "admin")
    public void createReviewTest() {
        List<String> reviewers = new ArrayList<>();
        reviewers.add("user@localhost");

        int size = reviewRepository.findAll().size();

        reviewService.createReview(new ReviewRequest("resource1", "codeability1.at", "2", reviewers));

        List<Review> reviewList = reviewRepository.findAll();
        assertThat(reviewList).hasSize(size + 1);

        Review testReview = reviewList.stream().filter(r -> r.getResource().equals("resource1")).findFirst().get();
        assertThat(testReview.getResource()).isEqualTo("resource1");
    }

    @Test
    @Transactional
    @WithMockUser(authorities = AuthoritiesConstants.USER, username = "user")
    public void updateReviewTest() {
        List<Review> reviewList = reviewRepository.findAll();
        Review testReview = reviewList.get(0);
        testReview.setLink("newLink");
        reviewService.updateReview(reviewService.reviewToDTO(testReview));

        reviewList = reviewRepository.findAll();
        assertThat(reviewList).hasSize(1);

        testReview = reviewList.get(0);
        assertThat(testReview.getLink()).isEqualTo("newLink");
    }

    @Test
    @Transactional
    @WithMockUser(authorities = AuthoritiesConstants.USER, username = "user")
    public void updateStatusTest() {
        List<Review> reviewList = reviewRepository.findAll();
        Review testReview = reviewList.get(0);

        reviewService.updateStatus(testReview.getId(), ReviewStatus.REVIEW_IN_PROGRESS);

        reviewList = reviewRepository.findAll();
        testReview = reviewList.get(0);
        testReview.getComments().forEach(c -> assertThat(c.getStatus()).isEqualTo(ReviewStatus.REVIEW_IN_PROGRESS));
    }

    @Test
    @Transactional
    @WithMockUser(authorities = AuthoritiesConstants.ADMIN, username = "admin")
    public void getAllReviewsTest() {
        List<String> reviewers = new ArrayList<>();
        reviewers.add("user@localhost");
        reviewService.createReview(new ReviewRequest("resource1", "codeability1.at", "2", reviewers));

        List<ReviewDTO> reviewList = reviewService.getAllReviews();
        assertThat(reviewList).hasSize(2);

        reviewList.forEach(r -> assertThat(r.getResource()).isIn("resource", "resource1"));
    }

    @Test
    @Transactional
    @WithMockUser(authorities = AuthoritiesConstants.ADMIN, username = "admin")
    public void getAllReviewsWithStatisticsTest() {
        List<ReviewDTO> reviewList = reviewService.getAllReviewsWithStatistics();
        assertThat(reviewList).hasSize(1);
        reviewList.forEach(r -> assertThat(r.getBadgeRewarded()).isFalse());
    }

    @Test
    @Transactional
    @WithMockUser(authorities = AuthoritiesConstants.USER, username = "admin")
    public void getReviewsByUserTest() {
        List<ReviewDTO> reviewList = reviewService.getReviewsByUser();
        reviewList.forEach(r -> assertThat(r.getResource()).isEqualTo("resource"));
        reviewList.forEach(r -> r.getUsers().stream().forEach(u -> assertEquals("ANONYMIZED", u)));
    }

    @Test
    @Transactional
    @WithMockUser(authorities = AuthoritiesConstants.USER, username = "user")
    public void updateReviewDeletionByUserTest() throws IllegalAccessException {
        List<ReviewDTO> reviewList = reviewService.getReviewsByUser();
        ReviewDTO reviewDTO = reviewList.get(0);

        reviewDTO.getStatus().set(0, "DELETION_REQUESTED");
        reviewService.updateReviewRatingByUser(reviewDTO);
        reviewList = reviewService.getReviewsByUser();
        assertEquals(reviewList.size(), 0);
    }

    @Test
    @Transactional
    @WithMockUser(authorities = AuthoritiesConstants.ADMIN, username = "admin")
    public void deleteReviewTest() {
        List<Review> reviewList = reviewRepository.findAll();
        Review testReview = reviewList.get(0);
        reviewService.deleteReview(testReview.getId());

        reviewList = reviewRepository.findAll();
        assertThat(reviewList).hasSize(0);
    }

    @Test
    @Transactional
    @Timeout(value = 10, unit = TimeUnit.MINUTES)
    @WithMockUser(authorities = AuthoritiesConstants.USER, username = "user")
    public void reviewToDTOTest() {
        // first create a new review request (without review rating)
        User loggedInUser = userService.getUserWithAuthorities().orElse(null);
        Review unstartedReview = new Review();
        unstartedReview.setLink("unused Link)");
        unstartedReview.setRequestedBy(loggedInUser.getId());
        unstartedReview.setResource("Some Resource to review");
        unstartedReview.setResourceID("resourceToReview");
        reviewRepository.save(unstartedReview);

        // first create another review request (without review rating, but with badge
        // rewarded)
        Review finishedReview = new Review();
        finishedReview.setLink("unused Link2)");
        finishedReview.setRequestedBy(loggedInUser.getId());
        finishedReview.setResource("Some Resource2 to review");
        finishedReview.setResourceID("resourceToReview2");
        reviewRepository.save(finishedReview);
        Statistics stat = new Statistics();
        stat.exerciseID("resourceToReview2").downloads(0).views(0).badgeRewarded(true);
        statisticsRepository.save(stat);
        statisticsRepository.flush();

        List<String> reviewers = new ArrayList<>();
        reviewers.add("user@localhost");

        Review testReview = reviewService.createReview(new ReviewRequest("resource2", "codeability2.at", "2", reviewers));
        reviewRepository.flush();

        ReviewDTO reviewDTO = reviewService.reviewToDTO(testReview);

        assertEquals(reviewDTO.getResource(), testReview.getResource());
        assertEquals(reviewDTO.getUsers(), testReview.getComments().stream().map(c -> c.getUser().getEmail()).collect(Collectors.toList()));
        assertEquals(reviewDTO.getComments(), testReview.getComments().stream().map(c -> c.getComment()).collect(Collectors.toList()));
        assertEquals(reviewDTO.getId(), testReview.getId());
        assertEquals(
            reviewDTO.getStatus(),
            testReview.getComments().stream().map(reviewRating -> reviewRating.getStatus().name()).collect(Collectors.toList())
        );
        assertEquals(reviewDTO.getLink(), testReview.getLink());

        Optional<at.ac.uibk.gitsearch.domain.User> userO = userService.getUserWithAuthoritiesByEmail("user@localhost");

        assertThat(userO).isPresent();
        final ReviewStatisticsDTO reviewStatusCountsByCurrentUser = reviewService.getReviewStatusCountsByUser(userO.get());
        assertThat(reviewStatusCountsByCurrentUser.getReviewInProgress())
            .withFailMessage("Status not as expected. This may result from changed testdata")
            .isEqualTo(2);

        // unfortunately in this test case reviewer and requesting user are the same
        // :-(
        assertThat(reviewStatusCountsByCurrentUser.getReviewInProgressByUser())
            .withFailMessage("Status not as expected. This may result from changed testdata")
            .isEqualTo(2);

        assertThat(reviewStatusCountsByCurrentUser.getBadgesRewarded())
            .withFailMessage("One badge awarded?. This may result from changed testdata")
            .isEqualTo(1);
    }

    @Test
    @Transactional
    @WithMockUser(authorities = AuthoritiesConstants.ADMIN, username = "admin")
    public void getReviewTest() {
        Review reviewTest = reviewRepository.findAll().get(0);

        ReviewDTO review = reviewService.getReview(reviewTest.getId());

        assertEquals(review.getId(), reviewTest.getId());
        assertEquals(review.getResource(), reviewTest.getResource());
        assertEquals(review.getLink(), reviewTest.getLink());
    }

    @Test
    @Transactional
    @WithMockUser(authorities = AuthoritiesConstants.ADMIN, username = "admin")
    public void rewardBadgeTest() {
        Review reviewTest = reviewRepository.findAll().get(0);
        reviewService.rewardBadge(reviewTest.getId());

        Statistics statistics = statisticsRepository.findByExerciseID(reviewTest.getResourceID()).get();

        assertEquals(statistics.getBadgeRewarded(), true);
    }

    @Test
    @Transactional
    @WithMockUser(authorities = AuthoritiesConstants.ADMIN, username = "admin")
    public void requestReviewForExerciseTest() {
        List<String> reviewers = new ArrayList<>();
        reviewers.add("user@localhost");

        reviewService.requestReviewForExercise(new ReviewRequest("resource3", "codeability3.at", "2", reviewers));

        List<Review> reviewList = reviewRepository.findAll();
        assertThat(reviewList).hasSize(2);

        Review testReview = reviewList.stream().filter(r -> r.getResource().equals("resource3")).findFirst().get();
        assertThat(testReview.getResource()).isEqualTo("resource3");
        testReview.getComments().forEach(c -> assertThat(c.getStatus()).isEqualTo("REQUESTED"));
    }
}
