package at.ac.uibk.gitsearch.web.rest;

import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.hasItem;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.delete;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.patch;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.put;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import at.ac.uibk.gitsearch.IntegrationTest;
import at.ac.uibk.gitsearch.domain.Review;
import at.ac.uibk.gitsearch.domain.ReviewHistoryEntry;
import at.ac.uibk.gitsearch.domain.ReviewRating;
import at.ac.uibk.gitsearch.domain.User;
import at.ac.uibk.gitsearch.domain.enumeration.ReviewStatus;
import at.ac.uibk.gitsearch.repository.ReviewRatingRepository;
import at.ac.uibk.gitsearch.security.AuthoritiesConstants;
import at.ac.uibk.gitsearch.service.dto.ReviewRatingDTO;
import at.ac.uibk.gitsearch.service.mapper.ReviewRatingMapper;
import java.util.List;
import java.util.Random;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicLong;
import javax.persistence.EntityManager;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.Timeout;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;

/**
 * Integration tests for the {@link ReviewRatingResource} REST controller.
 */
@IntegrationTest
@WithMockUser(authorities = AuthoritiesConstants.ADMIN)
@AutoConfigureMockMvc
class ReviewRatingResourceIT {

    private static final String DEFAULT_COMMENT = "AAAAAAAAAA";
    private static final String UPDATED_COMMENT = "BBBBBBBBBB";

    private static final ReviewStatus DEFAULT_STATUS = ReviewStatus.REVIEW_IN_PROGRESS;
    private static final ReviewStatus UPDATED_STATUS = ReviewStatus.IMPROVEMENTS_REQUESTED;

    private static final String ENTITY_API_URL = "/api/review-ratings";
    private static final String ENTITY_API_URL_ID = ENTITY_API_URL + "/{id}";

    private static Random random = new Random();
    private static AtomicLong count = new AtomicLong(random.nextInt() + (2 * Integer.MAX_VALUE));

    @Autowired
    private ReviewRatingRepository reviewRatingRepository;

    @Autowired
    private ReviewRatingMapper reviewRatingMapper;

    @Autowired
    private EntityManager em;

    @Autowired
    private MockMvc restReviewRatingMockMvc;

    private ReviewRating reviewRating;

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static ReviewRating createEntity(EntityManager em) {
        Review review = ReviewResourceIT.createEntity(em);
        // Add required entity
        User user = UserResourceIT.createEntity(em);
        em.persist(user);
        em.flush();

        ReviewRating reviewRating = new ReviewRating().comment(DEFAULT_COMMENT).status(DEFAULT_STATUS).review(review).user(user);
        return reviewRating;
    }

    /**
     * Create an updated entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static ReviewRating createUpdatedEntity(EntityManager em) {
        ReviewRating reviewRating = new ReviewRating().comment(UPDATED_COMMENT).status(UPDATED_STATUS);
        return reviewRating;
    }

    @BeforeEach
    public void initTest() {
        reviewRating = createEntity(em);
    }

    @Test
    @Transactional
    void createReviewRating() throws Exception {
        int databaseSizeBeforeCreate = reviewRatingRepository.findAll().size();
        // Create the ReviewRating
        ReviewRatingDTO reviewRatingDTO = reviewRatingMapper.toDto(reviewRating);
        restReviewRatingMockMvc
            .perform(
                post(ENTITY_API_URL)
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(reviewRatingDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isCreated());

        // Validate the ReviewRating in the database
        List<ReviewRating> reviewRatingList = reviewRatingRepository.findAll();
        assertThat(reviewRatingList).hasSize(databaseSizeBeforeCreate + 1);
        ReviewRating testReviewRating = reviewRatingList.get(reviewRatingList.size() - 1);
        assertThat(testReviewRating.getComment()).isEqualTo(DEFAULT_COMMENT);
        assertThat(testReviewRating.getStatus()).isEqualTo(DEFAULT_STATUS);
    }

    @Test
    @Transactional
    void createReviewRatingWithExistingId() throws Exception {
        // Create the ReviewRating with an existing ID
        reviewRating.setId(1L);
        ReviewRatingDTO reviewRatingDTO = reviewRatingMapper.toDto(reviewRating);

        int databaseSizeBeforeCreate = reviewRatingRepository.findAll().size();

        // An entity with an existing ID cannot be created, so this API call must fail
        restReviewRatingMockMvc
            .perform(
                post(ENTITY_API_URL)
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(reviewRatingDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isBadRequest());

        // Validate the ReviewRating in the database
        List<ReviewRating> reviewRatingList = reviewRatingRepository.findAll();
        assertThat(reviewRatingList).hasSize(databaseSizeBeforeCreate);
    }

    @Test
    @Transactional
    void getAllReviewRatings() throws Exception {
        // Initialize the database
        reviewRatingRepository.saveAndFlush(reviewRating);

        // Get all the reviewRatingList
        restReviewRatingMockMvc
            .perform(get(ENTITY_API_URL + "?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(reviewRating.getId().intValue())))
            .andExpect(jsonPath("$.[*].comment").value(hasItem(DEFAULT_COMMENT)))
            .andExpect(jsonPath("$.[*].status").value(hasItem(DEFAULT_STATUS.toString())));
    }

    @Test
    @Transactional
    void getReviewRating() throws Exception {
        // Initialize the database
        reviewRatingRepository.saveAndFlush(reviewRating);

        // Get the reviewRating
        restReviewRatingMockMvc
            .perform(get(ENTITY_API_URL_ID, reviewRating.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.id").value(reviewRating.getId().intValue()))
            .andExpect(jsonPath("$.comment").value(DEFAULT_COMMENT))
            .andExpect(jsonPath("$.status").value(DEFAULT_STATUS.toString()));
    }

    @Test
    @Transactional
    void getReviewRatingsByIdFiltering() throws Exception {
        // Initialize the database
        reviewRatingRepository.saveAndFlush(reviewRating);

        Long id = reviewRating.getId();

        defaultReviewRatingShouldBeFound("id.equals=" + id);
        defaultReviewRatingShouldNotBeFound("id.notEquals=" + id);

        defaultReviewRatingShouldBeFound("id.greaterThanOrEqual=" + id);
        defaultReviewRatingShouldNotBeFound("id.greaterThan=" + id);

        defaultReviewRatingShouldBeFound("id.lessThanOrEqual=" + id);
        defaultReviewRatingShouldNotBeFound("id.lessThan=" + id);
    }

    @Test
    @Transactional
    void getAllReviewRatingsByCommentIsEqualToSomething() throws Exception {
        // Initialize the database
        reviewRatingRepository.saveAndFlush(reviewRating);

        // Get all the reviewRatingList where comment equals to DEFAULT_COMMENT
        defaultReviewRatingShouldBeFound("comment.equals=" + DEFAULT_COMMENT);

        // Get all the reviewRatingList where comment equals to UPDATED_COMMENT
        defaultReviewRatingShouldNotBeFound("comment.equals=" + UPDATED_COMMENT);
    }

    @Test
    @Transactional
    void getAllReviewRatingsByCommentIsNotEqualToSomething() throws Exception {
        // Initialize the database
        reviewRatingRepository.saveAndFlush(reviewRating);

        // Get all the reviewRatingList where comment not equals to DEFAULT_COMMENT
        defaultReviewRatingShouldNotBeFound("comment.notEquals=" + DEFAULT_COMMENT);

        // Get all the reviewRatingList where comment not equals to UPDATED_COMMENT
        defaultReviewRatingShouldBeFound("comment.notEquals=" + UPDATED_COMMENT);
    }

    @Test
    @Transactional
    void getAllReviewRatingsByCommentIsInShouldWork() throws Exception {
        // Initialize the database
        reviewRatingRepository.saveAndFlush(reviewRating);

        // Get all the reviewRatingList where comment in DEFAULT_COMMENT or UPDATED_COMMENT
        defaultReviewRatingShouldBeFound("comment.in=" + DEFAULT_COMMENT + "," + UPDATED_COMMENT);

        // Get all the reviewRatingList where comment equals to UPDATED_COMMENT
        defaultReviewRatingShouldNotBeFound("comment.in=" + UPDATED_COMMENT);
    }

    @Test
    @Transactional
    void getAllReviewRatingsByCommentIsNullOrNotNull() throws Exception {
        // Initialize the database
        reviewRatingRepository.saveAndFlush(reviewRating);

        // Get all the reviewRatingList where comment is not null
        defaultReviewRatingShouldBeFound("comment.specified=true");

        // Get all the reviewRatingList where comment is null
        defaultReviewRatingShouldNotBeFound("comment.specified=false");
    }

    @Test
    @Transactional
    void getAllReviewRatingsByCommentContainsSomething() throws Exception {
        // Initialize the database
        reviewRatingRepository.saveAndFlush(reviewRating);

        // Get all the reviewRatingList where comment contains DEFAULT_COMMENT
        defaultReviewRatingShouldBeFound("comment.contains=" + DEFAULT_COMMENT);

        // Get all the reviewRatingList where comment contains UPDATED_COMMENT
        defaultReviewRatingShouldNotBeFound("comment.contains=" + UPDATED_COMMENT);
    }

    @Test
    @Timeout(value = 10, unit = TimeUnit.MINUTES)
    @Transactional
    void getAllReviewRatingsByCommentNotContainsSomething() throws Exception {
        // Initialize the database
        reviewRatingRepository.saveAndFlush(reviewRating);

        // Get all the reviewRatingList where comment does not contain DEFAULT_COMMENT
        defaultReviewRatingShouldNotBeFound("comment.doesNotContain=" + DEFAULT_COMMENT);

        // Get all the reviewRatingList where comment does not contain UPDATED_COMMENT
        defaultReviewRatingShouldBeFound("comment.doesNotContain=" + UPDATED_COMMENT);
    }

    @Test
    @Transactional
    void getAllReviewRatingsByStatusIsEqualToSomething() throws Exception {
        // Initialize the database
        reviewRatingRepository.saveAndFlush(reviewRating);

        // Get all the reviewRatingList where status equals to DEFAULT_STATUS
        defaultReviewRatingShouldBeFound("status.equals=" + DEFAULT_STATUS);

        // Get all the reviewRatingList where status equals to UPDATED_STATUS
        defaultReviewRatingShouldNotBeFound("status.equals=" + UPDATED_STATUS);
    }

    @Test
    @Transactional
    void getAllReviewRatingsByStatusIsNotEqualToSomething() throws Exception {
        // Initialize the database
        reviewRatingRepository.saveAndFlush(reviewRating);

        // Get all the reviewRatingList where status not equals to DEFAULT_STATUS
        defaultReviewRatingShouldNotBeFound("status.notEquals=" + DEFAULT_STATUS);

        // Get all the reviewRatingList where status not equals to UPDATED_STATUS
        defaultReviewRatingShouldBeFound("status.notEquals=" + UPDATED_STATUS);
    }

    @Test
    @Transactional
    void getAllReviewRatingsByStatusIsInShouldWork() throws Exception {
        // Initialize the database
        reviewRatingRepository.saveAndFlush(reviewRating);

        // Get all the reviewRatingList where status in DEFAULT_STATUS or UPDATED_STATUS
        defaultReviewRatingShouldBeFound("status.in=" + DEFAULT_STATUS + "," + UPDATED_STATUS);

        // Get all the reviewRatingList where status equals to UPDATED_STATUS
        defaultReviewRatingShouldNotBeFound("status.in=" + UPDATED_STATUS);
    }

    @Test
    @Transactional
    void getAllReviewRatingsByStatusIsNullOrNotNull() throws Exception {
        // Initialize the database
        reviewRatingRepository.saveAndFlush(reviewRating);

        // Get all the reviewRatingList where status is not null
        defaultReviewRatingShouldBeFound("status.specified=true");

        // Get all the reviewRatingList where status is null
        defaultReviewRatingShouldNotBeFound("status.specified=false");
    }

    @Test
    @Transactional
    void getAllReviewRatingsByReviewIsEqualToSomething() throws Exception {
        // Initialize the database
        reviewRatingRepository.saveAndFlush(reviewRating);
        Review review;
        if (TestUtil.findAll(em, Review.class).isEmpty()) {
            review = ReviewResourceIT.createEntity(em);
            em.persist(review);
            em.flush();
        } else {
            review = TestUtil.findAll(em, Review.class).get(0);
        }
        em.persist(review);
        em.flush();
        reviewRating.setReview(review);
        reviewRatingRepository.saveAndFlush(reviewRating);
        Long reviewId = review.getId();

        // Get all the reviewRatingList where review equals to reviewId
        defaultReviewRatingShouldBeFound("reviewId.equals=" + reviewId);

        // Get all the reviewRatingList where review equals to (reviewId + 1)
        defaultReviewRatingShouldNotBeFound("reviewId.equals=" + (reviewId + 1));
    }

    @Test
    @Transactional
    void getAllReviewRatingsByUserIsEqualToSomething() throws Exception {
        // Initialize the database
        reviewRatingRepository.saveAndFlush(reviewRating);
        User user;
        if (TestUtil.findAll(em, User.class).isEmpty()) {
            user = UserResourceIT.createEntity(em);
            em.persist(user);
            em.flush();
        } else {
            user = TestUtil.findAll(em, User.class).get(0);
        }
        em.persist(user);
        em.flush();
        reviewRating.setUser(user);
        reviewRatingRepository.saveAndFlush(reviewRating);
        Long userId = user.getId();

        // Get all the reviewRatingList where user equals to userId
        defaultReviewRatingShouldBeFound("userId.equals=" + userId);

        // Get all the reviewRatingList where user equals to (userId + 1)
        defaultReviewRatingShouldNotBeFound("userId.equals=" + (userId + 1));
    }

    @Test
    @Transactional
    void getAllReviewRatingsByHistoryIsEqualToSomething() throws Exception {
        // Initialize the database
        reviewRatingRepository.saveAndFlush(reviewRating);
        ReviewHistoryEntry history;
        if (TestUtil.findAll(em, ReviewHistoryEntry.class).isEmpty()) {
            history = ReviewHistoryEntryResourceIT.createEntity(em);
            em.persist(history);
            em.flush();
        } else {
            history = TestUtil.findAll(em, ReviewHistoryEntry.class).get(0);
        }
        em.persist(history);
        em.flush();
        reviewRating.addHistory(history);
        reviewRatingRepository.saveAndFlush(reviewRating);
        Long historyId = history.getId();

        // Get all the reviewRatingList where history equals to historyId
        defaultReviewRatingShouldBeFound("historyId.equals=" + historyId);

        // Get all the reviewRatingList where history equals to (historyId + 1)
        defaultReviewRatingShouldNotBeFound("historyId.equals=" + (historyId + 1));
    }

    /**
     * Executes the search, and checks that the default entity is returned.
     */
    private void defaultReviewRatingShouldBeFound(String filter) throws Exception {
        restReviewRatingMockMvc
            .perform(get(ENTITY_API_URL + "?sort=id,desc&" + filter))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(reviewRating.getId().intValue())))
            .andExpect(jsonPath("$.[*].comment").value(hasItem(DEFAULT_COMMENT)))
            .andExpect(jsonPath("$.[*].status").value(hasItem(DEFAULT_STATUS.toString())));

        // Check, that the count call also returns 1
        restReviewRatingMockMvc
            .perform(get(ENTITY_API_URL + "/count?sort=id,desc&" + filter))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(content().string("1"));
    }

    /**
     * Executes the search, and checks that the default entity is not returned.
     */
    private void defaultReviewRatingShouldNotBeFound(String filter) throws Exception {
        restReviewRatingMockMvc
            .perform(get(ENTITY_API_URL + "?sort=id,desc&" + filter))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$").isArray())
            .andExpect(jsonPath("$").isEmpty());

        // Check, that the count call also returns 0
        restReviewRatingMockMvc
            .perform(get(ENTITY_API_URL + "/count?sort=id,desc&" + filter))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(content().string("0"));
    }

    @Test
    @Transactional
    void getNonExistingReviewRating() throws Exception {
        // Get the reviewRating
        restReviewRatingMockMvc.perform(get(ENTITY_API_URL_ID, Long.MAX_VALUE)).andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    void putNewReviewRating() throws Exception {
        // Initialize the database
        reviewRatingRepository.saveAndFlush(reviewRating);

        int databaseSizeBeforeUpdate = reviewRatingRepository.findAll().size();

        // Update the reviewRating
        ReviewRating updatedReviewRating = reviewRatingRepository.findById(reviewRating.getId()).get();
        // Disconnect from session so that the updates on updatedReviewRating are not directly saved in db
        em.detach(updatedReviewRating);
        updatedReviewRating.comment(UPDATED_COMMENT).status(UPDATED_STATUS);
        ReviewRatingDTO reviewRatingDTO = reviewRatingMapper.toDto(updatedReviewRating);

        restReviewRatingMockMvc
            .perform(
                put(ENTITY_API_URL_ID, reviewRatingDTO.getId())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(reviewRatingDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isOk());

        // Validate the ReviewRating in the database
        List<ReviewRating> reviewRatingList = reviewRatingRepository.findAll();
        assertThat(reviewRatingList).hasSize(databaseSizeBeforeUpdate);
        ReviewRating testReviewRating = reviewRatingList.get(reviewRatingList.size() - 1);
        assertThat(testReviewRating.getComment()).isEqualTo(UPDATED_COMMENT);
        assertThat(testReviewRating.getStatus()).isEqualTo(UPDATED_STATUS);
    }

    @Test
    @Transactional
    void putNonExistingReviewRating() throws Exception {
        int databaseSizeBeforeUpdate = reviewRatingRepository.findAll().size();
        reviewRating.setId(count.incrementAndGet());

        // Create the ReviewRating
        ReviewRatingDTO reviewRatingDTO = reviewRatingMapper.toDto(reviewRating);

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restReviewRatingMockMvc
            .perform(
                put(ENTITY_API_URL_ID, reviewRatingDTO.getId())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(reviewRatingDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isBadRequest());

        // Validate the ReviewRating in the database
        List<ReviewRating> reviewRatingList = reviewRatingRepository.findAll();
        assertThat(reviewRatingList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithIdMismatchReviewRating() throws Exception {
        int databaseSizeBeforeUpdate = reviewRatingRepository.findAll().size();
        reviewRating.setId(count.incrementAndGet());

        // Create the ReviewRating
        ReviewRatingDTO reviewRatingDTO = reviewRatingMapper.toDto(reviewRating);

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restReviewRatingMockMvc
            .perform(
                put(ENTITY_API_URL_ID, count.incrementAndGet())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(reviewRatingDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isBadRequest());

        // Validate the ReviewRating in the database
        List<ReviewRating> reviewRatingList = reviewRatingRepository.findAll();
        assertThat(reviewRatingList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Timeout(value = 10, unit = TimeUnit.MINUTES)
    @Transactional
    void putWithMissingIdPathParamReviewRating() throws Exception {
        int databaseSizeBeforeUpdate = reviewRatingRepository.findAll().size();
        reviewRating.setId(count.incrementAndGet());

        // Create the ReviewRating
        ReviewRatingDTO reviewRatingDTO = reviewRatingMapper.toDto(reviewRating);

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restReviewRatingMockMvc
            .perform(
                put(ENTITY_API_URL)
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(TestUtil.convertObjectToJsonBytes(reviewRatingDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the ReviewRating in the database
        List<ReviewRating> reviewRatingList = reviewRatingRepository.findAll();
        assertThat(reviewRatingList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Timeout(value = 10, unit = TimeUnit.MINUTES)
    @Transactional
    void partialUpdateReviewRatingWithPatch() throws Exception {
        // Initialize the database
        reviewRatingRepository.saveAndFlush(reviewRating);

        int databaseSizeBeforeUpdate = reviewRatingRepository.findAll().size();

        // Update the reviewRating using partial update
        ReviewRating partialUpdatedReviewRating = new ReviewRating();
        partialUpdatedReviewRating.setId(reviewRating.getId());

        partialUpdatedReviewRating.status(UPDATED_STATUS);

        restReviewRatingMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedReviewRating.getId())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(partialUpdatedReviewRating))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isOk());

        // Validate the ReviewRating in the database
        List<ReviewRating> reviewRatingList = reviewRatingRepository.findAll();
        assertThat(reviewRatingList).hasSize(databaseSizeBeforeUpdate);
        ReviewRating testReviewRating = reviewRatingList.get(reviewRatingList.size() - 1);
        assertThat(testReviewRating.getComment()).isEqualTo(DEFAULT_COMMENT);
        assertThat(testReviewRating.getStatus()).isEqualTo(UPDATED_STATUS);
    }

    @Test
    @Transactional
    void fullUpdateReviewRatingWithPatch() throws Exception {
        // Initialize the database
        reviewRatingRepository.saveAndFlush(reviewRating);

        int databaseSizeBeforeUpdate = reviewRatingRepository.findAll().size();

        // Update the reviewRating using partial update
        ReviewRating partialUpdatedReviewRating = new ReviewRating();
        partialUpdatedReviewRating.setId(reviewRating.getId());

        partialUpdatedReviewRating.comment(UPDATED_COMMENT).status(UPDATED_STATUS);

        restReviewRatingMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedReviewRating.getId())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(partialUpdatedReviewRating))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isOk());

        // Validate the ReviewRating in the database
        List<ReviewRating> reviewRatingList = reviewRatingRepository.findAll();
        assertThat(reviewRatingList).hasSize(databaseSizeBeforeUpdate);
        ReviewRating testReviewRating = reviewRatingList.get(reviewRatingList.size() - 1);
        assertThat(testReviewRating.getComment()).isEqualTo(UPDATED_COMMENT);
        assertThat(testReviewRating.getStatus()).isEqualTo(UPDATED_STATUS);
    }

    @Test
    @Transactional
    void patchNonExistingReviewRating() throws Exception {
        int databaseSizeBeforeUpdate = reviewRatingRepository.findAll().size();
        reviewRating.setId(count.incrementAndGet());

        // Create the ReviewRating
        ReviewRatingDTO reviewRatingDTO = reviewRatingMapper.toDto(reviewRating);

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restReviewRatingMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, reviewRatingDTO.getId())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(reviewRatingDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isBadRequest());

        // Validate the ReviewRating in the database
        List<ReviewRating> reviewRatingList = reviewRatingRepository.findAll();
        assertThat(reviewRatingList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Timeout(value = 10, unit = TimeUnit.MINUTES)
    @Transactional
    void patchWithIdMismatchReviewRating() throws Exception {
        int databaseSizeBeforeUpdate = reviewRatingRepository.findAll().size();
        reviewRating.setId(count.incrementAndGet());

        // Create the ReviewRating
        ReviewRatingDTO reviewRatingDTO = reviewRatingMapper.toDto(reviewRating);

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restReviewRatingMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, count.incrementAndGet())
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(reviewRatingDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isBadRequest());

        // Validate the ReviewRating in the database
        List<ReviewRating> reviewRatingList = reviewRatingRepository.findAll();
        assertThat(reviewRatingList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Timeout(value = 10, unit = TimeUnit.MINUTES)
    @Transactional
    void patchWithMissingIdPathParamReviewRating() throws Exception {
        int databaseSizeBeforeUpdate = reviewRatingRepository.findAll().size();
        reviewRating.setId(count.incrementAndGet());

        // Create the ReviewRating
        ReviewRatingDTO reviewRatingDTO = reviewRatingMapper.toDto(reviewRating);

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restReviewRatingMockMvc
            .perform(
                patch(ENTITY_API_URL)
                    .contentType("application/merge-patch+json")
                    .content(TestUtil.convertObjectToJsonBytes(reviewRatingDTO))
                    .with(csrf().asHeader())
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the ReviewRating in the database
        List<ReviewRating> reviewRatingList = reviewRatingRepository.findAll();
        assertThat(reviewRatingList).hasSize(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void deleteReviewRating() throws Exception {
        // Initialize the database
        reviewRatingRepository.saveAndFlush(reviewRating);

        int databaseSizeBeforeDelete = reviewRatingRepository.findAll().size();

        // Delete the reviewRating
        restReviewRatingMockMvc
            .perform(delete(ENTITY_API_URL_ID, reviewRating.getId()).accept(MediaType.APPLICATION_JSON).with(csrf().asHeader()))
            .andExpect(status().isNoContent());

        // Validate the database contains one less item
        List<ReviewRating> reviewRatingList = reviewRatingRepository.findAll();
        assertThat(reviewRatingList).hasSize(databaseSizeBeforeDelete - 1);
    }
}
